# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets, QtCore, QtSvg, QtGui
import shiboken2


class BusyIndicator(QtWidgets.QWidget):
    '''Widget implementing busy indicator - a spinning wheel'''

    def __init__(self, start=True, parent=None):
        '''Initialise indicator with optional *parent*.'''
        super(BusyIndicator, self).__init__(parent=parent)
        self._timer = None
        self._timerInterval = 30
        self._speed = 8.0
        self._spinnerAngle = 0
        # TODO: Use properties to enable setting this colour via stylesheets.
        self._spinnerColor = QtGui.QColor(100, 87, 53)  # Color: '#FFDD86'
        self._logo = ':ftrack/image/default/ftrackLogoColor'
        if start:
            self.start()

    def start(self):
        '''Start spinning if not already.'''
        if self._timer is None:
            self._timer = self.startTimer(self._timerInterval)
        if not self.isVisible():
            self.setVisible(True)

    def stop(self):
        '''Stop spinning if currently spinning.'''
        if shiboken2.isValid(self) and self._timer is not None:
            self.killTimer(self._timer)
            self._timer = None

    def timerEvent(self, event):
        '''Handle timer *event*.'''
        self._spinnerAngle += self._speed
        if self._spinnerAngle >= 360.0:
            self._spinnerAngle = 0.0

        self.repaint()

    def paintEvent(self, event):
        '''Paint widget.'''
        painter = QtGui.QPainter()
        painter.begin(self)

        try:
            painter.setRenderHint(QtGui.QPainter.Antialiasing)
            area = QtCore.QRect(
                0, 0, painter.device().width(), painter.device().height()
            )
            center = QtCore.QPointF(area.width() / 2.0, area.height() / 2.0)

            # Draw in a normalised centered box.
            normalisedEdge = 100.0
            normalisedArea = QtCore.QRectF(
                -(normalisedEdge / 2.0),
                -(normalisedEdge / 2.0),
                normalisedEdge,
                normalisedEdge,
            )

            shortestSide = min(area.width(), area.height())
            painter.translate(center)
            painter.scale(
                shortestSide / normalisedEdge, shortestSide / normalisedEdge
            )

            # Draw spinner at current spin angle.
            pen = QtGui.QPen()
            penWidth = 8.0
            pen.setWidth(penWidth)
            pen.setColor(self._spinnerColor)
            pen.setCapStyle(QtCore.Qt.RoundCap)

            painter.setPen(pen)

            spinnerArea = QtCore.QRectF(
                normalisedArea.top() + (penWidth / 2.0),
                normalisedArea.left() + (penWidth / 2.0),
                normalisedArea.width() - penWidth,
                normalisedArea.height() - penWidth,
            )

            painter.drawArc(
                spinnerArea,
                -self._spinnerAngle * 16,  # Start angle.
                250 * 16,  # Span angle.
            )

        finally:
            painter.end()



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets, QtCore, QtGui

from ftrack_qt.utils.widget import set_property


class LineWidget(QtWidgets.QFrame):
    '''Widget presenting a one pixel wide line'''

    def __init__(self, horizontal=True, style=None, parent=None):
        '''
        Initialize Line

        :param horizontal: Line orientation
        :param style: The CSS style
        :param parent: The parent dialog or frame
        '''
        super(LineWidget, self).__init__(parent=parent)

        if horizontal:
            self.setMaximumHeight(1)
            self.setMinimumHeight(1)
            self.setSizePolicy(
                QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum
            )
        else:
            self.setMaximumWidth(1)
            self.setMinimumWidth(1)
            self.setMaximumHeight(16)
            self.setMinimumHeight(16)
            self.setSizePolicy(
                QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred
            )
        if style is not None:
            set_property(self, 'style', style)



from ftrack_qt.widgets.asset.asset_version_creation_widget import (
    AssetVersionCreation,
)
from ftrack_qt.widgets.asset.asset_version_selection_widget import (
    AssetVersionSelection,
)
from ftrack_qt.widgets.asset.new_asset_input_widget import NewAssetInput



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets, QtCore

from ftrack_qt.widgets.selectors import ListSelector
from ftrack_qt.widgets.headers import SessionHeader
from ftrack_qt.widgets.selectors import ContextSelector

from ftrack_qt.widgets.dialogs import StyledDialog


class TabDialog(StyledDialog):
    '''Base Class to represent a Plugin'''

    selected_context_changed = QtCore.Signal(object)
    selected_tab_changed = QtCore.Signal(object)

    @property
    def selected_context_id(self):
        '''Return the selected context id in the context selector'''
        return self._context_selector.context_id

    @selected_context_id.setter
    def selected_context_id(self, value):
        '''Set the given *value* as the selected context id'''
        if self.selected_context_id != value:
            self._context_selector.context_id = value

    @property
    def is_browsing_context(self):
        '''
        Return if context selector is currently working on setting up a context
        '''
        return self._context_selector.is_browsing

    def __init__(
        self,
        session,
        parent=None,
    ):
        '''
        Initialise BasePlugin with instance of
        :class:`ftrack_api.session.Session`
        '''
        super(TabDialog, self).__init__(parent=parent)

        self._session = session
        self._context_selector = None
        self._tab_widget = None
        self._header = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        main_layout = QtWidgets.QVBoxLayout()
        self.setLayout(main_layout)

    def build(self):
        # Create the header
        self._header = SessionHeader(self._session)

        # TODO: we have to update the signals from the context selector to
        #  identify that are our signals and not qt signals. So make them snake case
        self._context_selector = ContextSelector(
            self._session, enable_context_change=True
        )

        self._tab_widget = QtWidgets.QTabWidget()

        self.layout().addWidget(self._header)
        self.layout().addWidget(self._context_selector, QtCore.Qt.AlignTop)
        self.layout().addWidget(self._tab_widget)

    def post_build(self):
        '''Set up all the signals'''
        # Connect context selector signals
        self._context_selector.context_changed.connect(
            self._on_context_selected_callback
        )
        # Connect tab widget
        self._tab_widget.currentChanged.connect(self._on_tab_changed_callback)

    def add_tab(self, tab_title, widget):
        self._tab_widget.addTab(widget, tab_title)

    def _on_context_selected_callback(self, context_id):
        '''Emit signal with the new context_id'''
        if not context_id:
            return
        self.selected_context_changed.emit(context_id)

    def _on_tab_changed_callback(self, index):
        self.selected_tab_changed.emit(self._tab_widget.tabText(index))

    def clear_tab_ui(self):
        '''Remove all widgets from the tabs layout'''
        self._tab_widget.currentChanged.disconnect()
        while self._tab_widget.count():
            widget = self._tab_widget.widget(0)
            self._tab_widget.removeTab(0)
            if widget:
                widget.deleteLater()
        self._tab_widget.currentChanged.connect(self._on_tab_changed_callback)



from ftrack_qt.widgets.selectors.list_selector import ListSelector
from ftrack_qt.widgets.selectors.context_selector import ContextSelector
from ftrack_qt.widgets.selectors.asset_selector import (
    OpenAssetSelector,
    PublishAssetSelector,
)
from ftrack_qt.widgets.selectors.status_selector import StatusSelector
from ftrack_qt.widgets.selectors.version_selector import VersionSelector



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
# TODO: clean this code
import logging

from Qt import QtCore, QtWidgets, QtGui, QtSvg

import ftrack_constants as constants
from ftrack_qt.widgets.icons import MaterialIcon

logger = logging.getLogger(__name__)


# This is the old materialIconWidget
class StatusMaterialIconWidget(QtWidgets.QWidget):
    '''Material icon widget, support status > icon encoding'''

    @property
    def icon(self):
        '''Return the material icon'''
        return self._icon

    def __init__(self, name, variant=None, color=None, parent=None):
        '''
        Initialize MaterialIconWidget

        :param name: The name of material icon SVG image
        :param color: The color, in html #RRGGBB format, or rgba(r,g,b,alpha)
        :param variant: The variant of material icon to use
        :param parent:  The parent dialog or frame
        '''
        super(StatusMaterialIconWidget, self).__init__(parent=parent)
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(1)
        self._icon = None
        if name:
            self.set_icon(name, color=color, variant=variant)

    def set_icon(self, name, variant=None, color=None, size=16):
        '''Set the icon based on *name*, *variant*, *color* and *size*'''
        for i in reversed(range(self.layout().count())):
            self.layout().itemAt(i).widget().setParent(None)
        if color is None:
            color = 'gray'
        label = QtWidgets.QLabel()
        self._icon = MaterialIcon(name, variant=variant, color=color)
        label.setPixmap(self._icon.pixmap(QtCore.QSize(size, size)))
        self.layout().addWidget(label)

    def set_status(self, status, size=16):
        '''Set the icon based on the pipeline *status*'''
        icon_name = ''
        color = '303030'
        variant = 'filled'
        if status in [
            constants.status.UNKNOWN_STATUS,
            constants.status.DEFAULT_STATUS,
        ]:
            icon_name = 'help'
            color = '303030'
        elif status in [constants.status.RUNNING_STATUS]:
            icon_name = 'hourglass_bottom'
            color = '87E1EB'
        elif status in [constants.status.SUCCESS_STATUS]:
            icon_name = 'check-circle-outline'
            color = '79DFB6'
        elif status in [constants.status.WARNING_STATUS]:
            icon_name = 'error_outline'
            color = 'FFBD5D'
        elif status in [
            constants.status.ERROR_STATUS,
            constants.status.EXCEPTION_STATUS,
        ]:
            icon_name = 'error'
            color = 'FF7A73'
        self.set_icon(
            icon_name, variant=variant, color='#{}'.format(color), size=size
        )
        return color



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets, QtCore

from ftrack_qt.widgets.overlay import OverlayWidget
from ftrack_qt.widgets.lines import LineWidget
from ftrack_qt.utils.widget import (
    get_main_window_from_widget,
    get_framework_main_dialog,
)


class OptionsButton(QtWidgets.QPushButton):
    '''Options button on publisher accordion'''

    def __init__(self, title, icon, parent=None):
        '''
        Initialize options button

        :param title: The name of the step (component)
        :param icon: The button icon to use
        :param parent: the parent dialog or frame
        '''
        super(OptionsButton, self).__init__(parent=parent)
        self._title = title
        self._icon = icon
        self.__section_registry = {}

        self._main_widget = None
        self._options_widget = None
        self._overlay_container = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setMinimumSize(30, 30)
        self.setMaximumSize(30, 30)
        self.setIcon(self._icon)
        self.setFlat(True)

    def build(self):
        self._main_widget = QtWidgets.QFrame()
        self._main_widget.setLayout(QtWidgets.QVBoxLayout())
        self._main_widget.layout().setAlignment(QtCore.Qt.AlignTop)

        title_label = QtWidgets.QLabel(self._title)
        title_label.setObjectName('h2')
        self._main_widget.layout().addWidget(title_label)
        self._main_widget.layout().addWidget(QtWidgets.QLabel(''))

        self._options_widget = QtWidgets.QWidget()
        self._options_widget.setLayout(QtWidgets.QVBoxLayout())
        self._options_widget.layout().addWidget(
            QtWidgets.QLabel(''), 100
        )  # spacer

        scroll = QtWidgets.QScrollArea()
        scroll.setWidget(self._options_widget)
        scroll.setWidgetResizable(True)
        scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        scroll.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)

        self._main_widget.layout().addWidget(scroll)
        self._overlay_container = OverlayWidget(
            self._main_widget, height_percentage=0.9
        )
        self._overlay_container.setVisible(False)

    def post_build(self):
        self.clicked.connect(self.on_click_callback)

    def on_click_callback(self):
        '''Callback on clicking the options button, show the publish options overlay'''
        # Check first if widget is running on a ftrack framework dialog
        main_window = get_framework_main_dialog(self)
        if not main_window:
            main_window = get_main_window_from_widget(self)
        if main_window:
            self._overlay_container.setParent(main_window)
        self._overlay_container.setVisible(True)

    def add_widget(self, widget, section_name):
        if section_name not in self.__section_registry:
            self._options_widget.layout().insertWidget(
                self._options_widget.layout().count() - 1, LineWidget()
            )
            section_label = QtWidgets.QLabel("{}:".format(section_name))
            section_label.setObjectName('gray')
            self._options_widget.layout().insertWidget(
                self._options_widget.layout().count() - 1,
                section_label,
            )
            section_widget = QtWidgets.QWidget()
            section_widget_layout = QtWidgets.QVBoxLayout()
            section_widget.setLayout(section_widget_layout)
            self._options_widget.layout().insertWidget(
                self._options_widget.layout().count() - 1, section_widget
            )

            # TODO: create the section Widget
            self.__section_registry[section_name] = section_widget

        self.__section_registry[section_name].layout().insertWidget(
            self._options_widget.layout().count() - 1, LineWidget()
        )
        self.__section_registry[section_name].layout().insertWidget(
            self._options_widget.layout().count() - 1, widget
        )



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets, QtCore, QtGui

from ftrack_qt.widgets.models import TableModel


class TableView(QtWidgets.QTableView):
    '''Generic table view to represent an item.'''

    # This is to be re-implemented in the children
    selection_changed = QtCore.Signal(object)

    @property
    def table_model(self):
        return self._generic_model

    def __init__(self, column_mapping=None, parent=None):
        '''Initialise TableView with *column_mapping*

        *column_mapping*: Is a dictionary to map data item keys to header titles
        '''
        super(TableView, self).__init__(parent=parent)

        self._column_mapping = column_mapping
        self._generic_model = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        '''Prepare general layout.'''

        self.verticalHeader().hide()

        self.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)

        self.horizontalHeader().setStretchLastSection(True)

    def build(self):
        '''Build widgets and parent them.'''
        self._generic_model = TableModel(
            column_mapping=self._column_mapping, parent=self
        )

        self.setModel(self._generic_model)

    def post_build(self):
        '''Perform post-construction operations.'''
        pass

    def set_data_items(self, data_items):
        '''
        Sets the :obj:`data_items` with the given *data_items*
        '''
        self.table_model.set_data_items(data_items)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets, QtCore, QtGui

from ftrack_qt.widgets.thumbnails import AssetVersionThumbnail


class AssetVersionCreation(QtWidgets.QFrame):
    '''Widget representing the next version of the asset version'''

    @property
    def version(self):
        '''Return the latest version of the asset.'''
        if not self._versions:
            return None
        return self._versions[-1]

    @property
    def asset_id(self):
        '''Return the id of the asset.'''
        return self._asset_id

    @property
    def active(self):
        '''Return the active state of the widget.'''
        return self._active

    @active.setter
    def active(self, value):
        '''Change active state of the widget - greys out the widget if False.'''
        if value == self._active:
            return
        self._active = value
        self._asset_name_widget.style().unpolish(self._asset_name_widget)
        self._create_label.style().unpolish(self._create_label)
        self._version_label.style().unpolish(self._version_label)
        if value:
            self._asset_name_widget.setObjectName('')
            self._create_label.setObjectName('gray')
            self._version_label.setObjectName('color-primary')
        else:
            self._asset_name_widget.setObjectName('gray-dark')
            self._create_label.setObjectName('gray-darker')
            self._version_label.setObjectName('gray-darker')
        self._asset_name_widget.style().polish(self._asset_name_widget)
        self._create_label.style().polish(self._create_label)
        self._version_label.style().polish(self._version_label)

    def __init__(self, asset_name, asset_id, versions, server_url):
        '''Initialize the AssetVersionCreation widget.'''
        super(AssetVersionCreation, self).__init__()

        self._asset_id = asset_id
        self._asset_name = asset_name
        self._versions = versions
        self._server_url = server_url
        self._active = True

        self._thumbnail_widget = None
        self._asset_name_widget = None
        self._create_label = None
        self._version_label = None

        self.pre_build()
        self.build()

    def pre_build(self):
        '''Set up the layout for the widget.'''
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(5)

    def build(self):
        '''Build the widget components.'''
        self._thumbnail_widget = AssetVersionThumbnail()
        self._thumbnail_widget.setScaledContents(True)
        self._thumbnail_widget.setMinimumSize(57, 31)
        self._thumbnail_widget.setMaximumSize(57, 31)
        self._thumbnail_widget.set_server_url(self._server_url)
        self._thumbnail_widget.load(
            self.version['thumbnail'] if self.version else None
        )
        self.layout().addWidget(self._thumbnail_widget)

        self._asset_name_widget = QtWidgets.QLabel(self._asset_name)
        self.layout().addWidget(self._asset_name_widget)

        self._create_label = QtWidgets.QLabel('- create')
        self._create_label.setObjectName('gray')
        self.layout().addWidget(self._create_label)

        self._version_label = QtWidgets.QLabel(
            'Version {}'.format(
                self.version['next_version'] if self.version else "1"
            )
        )
        self._version_label.setObjectName('color-primary')
        self.layout().addWidget(self._version_label)

        self.layout().addStretch()

        self.setToolTip(self._asset_name)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
import sys

from Qt import QtCore, QtWidgets, QtGui

import ftrack_constants.qt as qt_constants


def apply_font(font=':/ftrack/font/main'):
    '''Add application *font*.'''
    QtGui.QFontDatabase.addApplicationFont(font)


def apply_theme(widget, theme=None):
    '''Apply *theme* to *widget* - load stylesheet from resource file and apply'''
    apply_font()
    if theme is None:
        theme = qt_constants.theme.DEFAULT_THEME
    theme_path = ':/ftrack/style/{0}'.format(theme)
    fileObject = QtCore.QFile(theme_path)
    if fileObject.exists():
        fileObject.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text)
        stream = QtCore.QTextStream(fileObject)
        styleSheetContent = stream.readAll()

        widget.setStyleSheet(styleSheetContent)
    else:
        sys.stderr.write(
            'ftrack theme "{}" could not be found! Make sure to import '
            'resource.py from ftrack_qt_style library.\n'.format(theme_path)
        )



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtCore, QtWidgets, QtGui

from ftrack_qt.widgets.logos import FtrackLogo as Logo
from ftrack_qt.widgets.user import FtrackUser as User
from ftrack_qt.utils.layout import recursive_clear_layout


class SessionHeader(QtWidgets.QFrame):
    '''Header widget with name and thumbnail'''

    def __init__(
        self,
        session,
        title=None,
        show_logo=True,
        show_user=True,
        parent=None,
    ):
        '''
        Instantiate the header widget

        :param session: :class:`ftrack_api.session.Session` instance
        :param title: (Optional) The title to put in header
        :param show_logo: If True, Connect logo should be displayed
        :param show_user: If True, the user avatar icon should be displayed
        :param parent: The parent dialog or frame
        '''
        super(SessionHeader, self).__init__(parent=parent)

        self.session = session
        self._show_logo = show_logo
        self._title = title
        self._show_user = show_user
        self.setObjectName('ftrack-header-widget')

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setLayout(QtWidgets.QVBoxLayout())
        self.layout().setContentsMargins(8, 2, 7, 8)
        self.layout().setAlignment(QtCore.Qt.AlignTop)

    def build(self):
        # Logo & User ID
        self.id_container = QtWidgets.QWidget(self)
        self.id_container_layout = QtWidgets.QHBoxLayout()
        self.id_container_layout.setContentsMargins(1, 1, 1, 1)
        self.id_container_layout.setSpacing(5)
        self.id_container_layout.setAlignment(QtCore.Qt.AlignTop)
        self.id_container.setLayout(self.id_container_layout)

        self.logo = Logo(self)
        if len(self._title or ''):
            self._title_label = QtWidgets.QLabel(self._title)
            self._title_label.setObjectName('h2')
        self.content_container = QtWidgets.QWidget()
        self.content_container.setLayout(QtWidgets.QHBoxLayout())
        self.content_container.layout().setContentsMargins(0, 0, 0, 0)
        self.content_container.layout().setSpacing(0)

        self.user = User(self.session)

        self.id_container_layout.addWidget(self.logo)
        if len(self._title or ''):
            self.id_container_layout.addWidget(self._title_label)
        self.id_container_layout.addWidget(self.content_container, 1000)
        self.id_container_layout.addWidget(self.user)

        # Add (Logo & User ID) & Message
        self.layout().addWidget(self.id_container)

    def post_build(self):
        self.logo.setVisible(self._show_logo)
        self.user.setVisible(self._show_user)

    def add_widget(self, widget):
        self.content_container.layout().addWidget(widget)

    def set_widget(self, widget):
        recursive_clear_layout(self.content_container.layout())
        self.add_widget(widget)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtCore, QtWidgets


class InvokeEvent(QtCore.QEvent):
    '''Event.'''

    def __init__(self, fn, *args, **kwargs):
        '''Invoke *fn* in main thread.'''
        QtCore.QEvent.__init__(
            self, QtCore.QEvent.Type(QtCore.QEvent.registerEventType())
        )
        self.fn = fn
        self.args = args
        self.kwargs = kwargs


class Invoker(QtCore.QObject):
    '''Invoker.'''

    def event(self, event):
        '''Call function on *event*.'''
        event.fn(*event.args, **event.kwargs)

        return True


_invoker = Invoker(None)


def invoke_in_qt_main_thread(fn, *args, **kwargs):
    '''
    Invoke function *fn* with arguments, if not running in the main thread.

    TODO: Align this with DCC utility functions to run in main thread, and use them
    instead as their QT implementation might differ and this solution might not apply
    or cause instabilities.
    '''
    if QtCore.QThread.currentThread() is _invoker.thread():
        fn(*args, **kwargs)
    else:
        QtCore.QCoreApplication.postEvent(
            _invoker, InvokeEvent(fn, *args, **kwargs)
        )



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

import os
import logging

# Evaluate version and log package version
try:
    from ftrack_utils.version import get_version

    __version__ = get_version(
        os.path.basename(os.path.dirname(__file__)),
        os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
    )
except Exception:
    __version__ = "0.0.0"

logger = logging.getLogger(__name__)
logger.debug("v{}".format(__version__))

# DO NOT REMOVE UNUSED IMPORT - important to keep this in order to have resources
# initialised properly for applying style and providing images & fonts.

try:
    from ftrack_qt_style import (
        resource,
    )
except Exception:
    logger.exception(
        "Styling resource file not found, please install ftrack-qt-style library"
    )

# TODO: review all the code inside this library, we should standardized docstrings.



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

import logging

from Qt import QtWidgets, QtCore, QtGui

from ftrack_qt.widgets.asset import (
    AssetVersionCreation,
    AssetVersionSelection,
    NewAssetInput,
)
from ftrack_qt.widgets.lists import AssetList


class AssetSelectorBase(QtWidgets.QWidget):
    '''This widget allows the user to select an existing asset and asset version.'''

    assets_added = QtCore.Signal(object)
    '''This signal is emitted when assets are added. It sends a list of assets 
    dictionaries as an argument'''

    version_changed = QtCore.Signal(object)
    '''This signal is emitted when the version is changed. It sends the version 
    number as an argument '''

    selected_item_changed = QtCore.Signal(object, object)
    '''This signal is emitted when an item is selected. It sends the version 
    dictionary as an argument and the asset id'''

    def __init__(
        self,
        parent=None,
    ):
        '''
        This method initialises the asset selector widget.
        '''
        super(AssetSelectorBase, self).__init__(parent=parent)
        self.logger = logging.getLogger(
            __name__ + '.' + self.__class__.__name__
        )

        self._asset_list = None

        self.selected_index = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        '''This method sets up the main layout for the widget.'''
        main_layout = QtWidgets.QVBoxLayout()
        main_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(main_layout)

    def build(self):
        '''This method builds the asset list and adds it to the layout.'''

        self._asset_list = AssetList(AssetVersionSelection)

        self.layout().addWidget(self._asset_list)

    def post_build(self):
        '''This method connects signals to slots after building the widget.'''
        self._asset_list.assets_added.connect(self._on_assets_added)
        self._asset_list.version_changed.connect(self._on_version_changed)
        self._asset_list.selected_item_changed.connect(
            self._on_selected_item_changed_callback
        )

    def _on_assets_added(self, assets):
        '''This method emits the assets_added signal with the given assets.'''
        self.assets_added.emit(assets)

    def set_assets(self, assets):
        '''This method sets the assets in the asset list and shows or hides it
        based on the presence of assets.'''
        self._asset_list.set_assets(assets)

    def _on_version_changed(self, version):
        '''This method emits the version_changed signal with the given version.'''
        self.version_changed.emit(version)

    def _on_selected_item_changed_callback(self, index, version, asset_id):
        '''This method updates the selected index and emits the
        selected_item_changed signal with the given version.'''
        self.selected_index = index
        self.selected_item_changed.emit(version, asset_id)


class OpenAssetSelector(AssetSelectorBase):
    '''Asset selector tailored for open.'''

    def __init__(self):
        '''This method initialises the open asset selector widget.'''
        super(OpenAssetSelector, self).__init__()


class PublishAssetSelector(AssetSelectorBase):
    '''Asset selector tailored for publish, allows user to select and existing
    asset or input an asset name for creating a new asset.'''

    VALID_ASSET_NAME = QtCore.QRegExp('[A-Za-z0-9_]+')

    new_asset = QtCore.Signal(object)
    '''This signal is emitted when a new asset is selected. It sends the
    new asset_name as argument.'''

    def __init__(
        self,
        parent=None,
    ):
        '''
        This method initialises the publish asset selector widget.
        '''
        self._list_and_input = None
        self._new_asset_input = None
        self.validator = QtGui.QRegExpValidator(self.VALID_ASSET_NAME)
        self.placeholder_name = "Asset Name..."

        super(PublishAssetSelector, self).__init__(parent=parent)

    def build(self):
        '''This method builds the asset list and new asset input and adds them
        to the layout.'''
        self._list_and_input = AssetListAndInput()

        self._asset_list = AssetList(AssetVersionCreation)

        self._list_and_input.add_asset_list(self._asset_list)

        # Create new asset
        self._new_asset_input = NewAssetInput(
            self.validator, self.placeholder_name
        )
        self._list_and_input.add_asset_input(self._new_asset_input)

        self.layout().addWidget(self._list_and_input)

    def post_build(self):
        '''This method connects signals to slots after building the widget.'''
        super(PublishAssetSelector, self).post_build()
        self._asset_list.active_changed.connect(
            self._on_asset_list_active_changed_callback
        )
        self._new_asset_input.active_changed.connect(
            self._on_new_asset_active_changed_callback
        )
        self._new_asset_input.text_changed.connect(
            self._on_new_asset_name_changed_callback
        )

    def _on_asset_list_active_changed_callback(self, active):
        if active:
            # Deactivate new input
            self._new_asset_input.active = False

    def _on_new_asset_active_changed_callback(self, active):
        '''This method handles changes to the new asset name input.'''
        if active:
            # Deactivate list
            self._asset_list.active = False
            self.selected_index = None
            self.selected_item_changed.emit(None, None)

    def _on_new_asset_name_changed_callback(self, asset_name):
        is_valid_name = self.validate_name(asset_name)
        if is_valid_name:
            self.new_asset.emit(asset_name)
        else:
            self.new_asset.emit(None)

    def validate_name(self, asset_name):
        '''This method validates the given asset name and updates the input style.'''
        is_valid_bool = True
        # Already an asset by that name
        if self._asset_list.assets:
            for asset_id, asset_dict in self._asset_list.assets.items():
                if asset_dict['name'].lower() == asset_name.lower():
                    is_valid_bool = False
                    break
        if is_valid_bool and self.validator:
            is_valid = self.validator.validate(asset_name, 0)
            if is_valid[0] != QtGui.QValidator.Acceptable:
                is_valid_bool = False
            else:
                is_valid_bool = True
        self._new_asset_input.set_valid(is_valid_bool)
        return is_valid_bool

    def set_default_new_asset_name(self, name):
        '''This method sets the default name for the new asset input.'''
        self._new_asset_input.set_default_name(name)

    def set_assets(self, assets):
        super(PublishAssetSelector, self).set_assets(assets)
        if not assets:
            self._asset_list.hide()
            self._new_asset_input.active = True
        else:
            self._asset_list.show()
            self._new_asset_input.active = False
        # Make sure widget expands properly to fit list
        self._list_and_input.size_changed()


class AssetListAndInput(QtWidgets.QWidget):
    '''This is a compound widget containing an asset list and a new asset input'''

    def __init__(self, parent=None):
        '''This method initializes the widget with a vertical layout.'''
        super(AssetListAndInput, self).__init__(parent=parent)
        self.setLayout(QtWidgets.QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)
        self._asset_list = None
        self._asset_input = None

    def add_asset_list(self, asset_list):
        '''This method adds the given asset list to the widget.'''
        self._asset_list = asset_list
        self.layout().addWidget(asset_list)

    def add_asset_input(self, asset_input):
        '''This method adds the given asset input to the widget.'''
        self._asset_input = asset_input
        self.layout().addWidget(asset_input)

    def resizeEvent(self, event):
        '''This method overrides the resize event to handle size changes.'''
        self.size_changed()

    def size_changed(self):
        '''This method resizes the asset list to fit the widget, preventing
        unnecessary scrolling.'''
        height = self.size().height()
        optimal_height = (
            self._asset_list.sizeHintForRow(0) * self._asset_list.count()
        )
        if height != optimal_height:
            self._asset_list.setFixedHeight(optimal_height)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets

import ftrack_constants as constants
from ftrack_qt.utils.widget import set_property
from ftrack_qt.widgets.icons import StatusMaterialIconWidget
from ftrack_qt.widgets.overlay import OverlayWidget


class ProgressStatusButtonWidget(QtWidgets.QPushButton):
    '''Progress main status button representation, to be put inside the
    tool header.'''

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    def __init__(self, style_mode, parent=None):
        super(ProgressStatusButtonWidget, self).__init__(parent=parent)

        self._status = constants.status.UNKNOWN_STATUS
        self._style_mode = style_mode

        self._message_label = None
        self._status_icon = None

        self.pre_build()
        self.build()

    def pre_build(self):
        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(6, 6, 6, 6)
        layout.setSpacing(1)
        self.setLayout(layout)

        self.setMinimumHeight(32)
        self.setMinimumWidth(200)

    def build(self):
        self.setObjectName(f'progress-widget-{self._style_mode}')

        self._message_label = QtWidgets.QLabel()
        self.layout().addWidget(self._message_label)
        self.layout().addStretch()
        self._status_icon = StatusMaterialIconWidget('')
        self.layout().addWidget(self._status_icon)

        self.set_status(constants.status.UNKNOWN_STATUS)

    def set_status(self, new_status, message=''):
        assert (
            new_status in constants.status.status_bool_mapping.keys()
        ), f'Invalid status: {new_status}'
        self.status = new_status
        if message:
            self._message_label.setText(message)
        set_property(self, 'status', self.status.lower())
        color = self._status_icon.set_status(self.status, size=24)
        self._message_label.setStyleSheet(f'color: #{color}')


class ProgressPhaseButtonWidget(QtWidgets.QPushButton):
    '''Showing progress of a progress phase, when pressed the log is shown.'''

    @property
    def category(self):
        return self._category

    @property
    def label(self):
        return self._label

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def log_message(self):
        return self._log_message

    @log_message.setter
    def log_message(self, value):
        '''Store for log *value* for view and set it as tooltip'''
        self._log_message = value
        self.setToolTip(value or '')

    def __init__(self, label, category='', tags=None, parent=None):
        '''Instantiate the PhaseButtonWidget with *label* and *status*'''

        super(ProgressPhaseButtonWidget, self).__init__(parent=parent)

        self._label = label
        self._category = category
        self._tags = tags or []
        self._status = constants.status.UNKNOWN_STATUS
        self._log_message = None

        self._log_overlay_container = None
        self._icon_widget = None
        self._status_message_widget = None
        self._log_message_widget = None
        self._log_text_edit = None
        self._close_button = None
        self._time_widget = None
        self.log_overlay_container = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        layout = QtWidgets.QHBoxLayout()
        self.setLayout(layout)
        self.setMinimumHeight(
            48
        )  # Set minimum otherwise it will collapse the container
        self.setMinimumWidth(200)
        self.layout().setContentsMargins(3, 3, 3, 3)

    def build(self):
        self._icon_widget = StatusMaterialIconWidget(None)
        self.layout().addWidget(self._icon_widget)
        self.set_status(self.status)

        v_layout = QtWidgets.QVBoxLayout()

        label_widget = QtWidgets.QLabel(self._label)
        label_widget.setObjectName('h3')
        v_layout.addWidget(label_widget)

        # Show tags as chips
        h_layout = QtWidgets.QHBoxLayout()
        h_layout.setContentsMargins(0, 0, 0, 0)

        for tag in self._tags:
            tag_widget = QtWidgets.QLabel(tag)
            tag_widget.setObjectName('gray')
            tag_widget.setStyleSheet(
                'background: #333333; padding: 1px; border-radius: 6px;'
            )
            h_layout.addWidget(tag_widget)
        h_layout.addWidget(QtWidgets.QLabel(''), 100)  # Add spacing

        v_layout.addLayout(h_layout)

        self.layout().addLayout(v_layout, 100)

        v_layout = QtWidgets.QVBoxLayout()

        self._status_message_widget = QtWidgets.QLabel(self.status)
        self._status_message_widget.setObjectName('gray')
        v_layout.addWidget(self._status_message_widget)

        self._time_widget = QtWidgets.QLabel()
        self._time_widget.setObjectName('gray')
        v_layout.addWidget(self._time_widget)

        self.layout().addLayout(v_layout)

        self._log_message_widget = QtWidgets.QFrame()
        self._log_message_widget.setLayout(QtWidgets.QVBoxLayout())
        self._log_message_widget.layout().addSpacing(5)
        self._log_message_widget.layout().addWidget(
            QtWidgets.QLabel(f'{self.label} log:')
        )

        self._log_text_edit = QtWidgets.QTextEdit()
        self._log_text_edit.setReadOnly(True)
        self._log_message_widget.layout().addWidget(self._log_text_edit, 100)
        self._close_button = QtWidgets.QPushButton('HIDE LOG')
        self._log_message_widget.layout().addWidget(self._close_button)
        self.log_overlay_container = OverlayWidget(
            self._log_message_widget,
            transparent_background=False,
        )
        self.log_overlay_container.setVisible(False)

    def post_build(self):
        self.clicked.connect(self.show_log)
        self._close_button.clicked.connect(self.hide_log)

    def update_status(
        self, new_status, status_message, log_message, time=None
    ):
        '''Update the status of the phase to *new_status* and set *status_message*. 
        Build log messages from *log*.''' ''
        color = self.set_status(new_status)
        self._status_message_widget.setText(f'[{status_message.upper()}]')
        self._status_message_widget.setStyleSheet(f'color: #{color};')
        if time:
            self._time_widget.setText(f'{time:.3f}s')
        else:
            self._time_widget.setText('')
        self.log_message = log_message

    def set_status(self, new_status):
        '''Visualize *new_status* on the button'''
        self.status = new_status
        set_property(self, 'status', self.status.lower())
        return self._icon_widget.set_status(self.status)

    def show_log(self):
        self.log_overlay_container.setParent(self.parent())
        if self.log_message:
            self._log_text_edit.setText(self.log_message)
        else:
            self._log_text_edit.setText("No errors found")
        self.log_overlay_container.setVisible(True)
        self.log_overlay_container.resize(self.parent().size())

    def hide_log(self):
        self.log_overlay_container.setVisible(False)

    def teardown(self):
        '''Teardown the progress button - close the overlay container'''
        self.hide_log()
        self.log_overlay_container.deleteLater()



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from ftrack_qt.widgets.overlay.busy_indicator_widget import BusyIndicator
from ftrack_qt.widgets.overlay.shaded_widget import ShadedWidget
from ftrack_qt.widgets.overlay.overlay_widget import OverlayWidget



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from Qt import QtWidgets, QtCore

from ftrack_qt.widgets.selectors import ListSelector
from ftrack_qt.widgets.headers import SessionHeader
from ftrack_qt.widgets.selectors import ContextSelector

from ftrack_qt.widgets.dialogs import StyledDialog

from ftrack_qt.utils.layout import recursive_clear_layout


class ScrollToolConfigsDialog(StyledDialog):
    '''Base Class to represent a Plugin'''

    selected_context_changed = QtCore.Signal(object)
    selected_tool_config_changed = QtCore.Signal(object)
    refresh_tool_configs_clicked = QtCore.Signal()
    run_button_clicked = QtCore.Signal()

    @property
    def tool_config_widget(self):
        '''Return the tool config widget of the dialog'''
        return self._tool_config_widget

    @property
    def run_button_title(self):
        '''Return the label of the run button'''
        return self._run_button_title

    @run_button_title.setter
    def run_button_title(self, value):
        '''Set the label of the run button'''
        self._run_button_title = str(value)

    @property
    def selected_context_id(self):
        '''Return the selected context id in the context selector'''
        return self._context_selector.context_id

    @selected_context_id.setter
    def selected_context_id(self, value):
        '''Set the given *value* as the selected context id'''
        if self.selected_context_id != value:
            self._context_selector.context_id = value

    @property
    def is_browsing_context(self):
        '''
        Return if context selector is currently working on setting up a context
        '''
        return self._context_selector.is_browsing

    @property
    def selected_tool_config_name(self):
        '''Return the selected tool config name'''
        if self._tool_config_selector.current_item_index() in [0, -1]:
            return None
        return self._tool_config_selector.current_item_text()

    @selected_tool_config_name.setter
    def selected_tool_config_name(self, value):
        '''Set the given *value* as the selected tool config name'''
        if not self.selected_tool_config_name and not value:
            self.clear_tool_config_ui()
        if self.selected_tool_config_name != value:
            self.clear_tool_config_ui()
            if not value:
                self._tool_config_selector.set_current_item_index(0)
                return
            self._tool_config_selector.set_current_item(value)

    def __init__(
        self,
        session,
        parent=None,
    ):
        '''
        Initialise BasePlugin with instance of
        :class:`ftrack_api.session.Session`
        '''
        super(ScrollToolConfigsDialog, self).__init__(parent=parent)

        self._session = session
        self._context_selector = None
        self._tool_config_selector = None
        self._header = None
        self._scroll_area = None
        self._tool_config_widget = None
        self._run_button = None
        self._run_button_title = 'Run'

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        main_layout = QtWidgets.QVBoxLayout()
        self.setLayout(main_layout)

    def build(self):
        # Create the header
        self._header = SessionHeader(self._session)

        self._context_selector = ContextSelector(
            self._session, enable_context_change=True
        )

        self._tool_config_selector = ListSelector("Configs")

        self._scroll_area = QtWidgets.QScrollArea()
        self._scroll_area.setStyle(QtWidgets.QStyleFactory.create("plastique"))
        self._scroll_area.setWidgetResizable(True)
        self._scroll_area.setHorizontalScrollBarPolicy(
            QtCore.Qt.ScrollBarAlwaysOff
        )

        self._tool_config_widget = QtWidgets.QWidget()
        _tool_config_widget_layout = QtWidgets.QVBoxLayout()
        self._tool_config_widget.setLayout(_tool_config_widget_layout)

        self._run_button = QtWidgets.QPushButton(self._run_button_title)

        self.layout().addWidget(self._header)
        self.layout().addWidget(self._context_selector, QtCore.Qt.AlignTop)
        self.layout().addWidget(self._tool_config_selector)
        self.layout().addWidget(self._scroll_area, 100)
        self._scroll_area.setWidget(self._tool_config_widget)
        self.layout().addWidget(self._run_button)

    def post_build(self):
        '''Set up all the signals'''
        # Connect context selector signals
        self._context_selector.context_changed.connect(
            self._on_context_selected_callback
        )
        # Connect tool_config selector signals
        self._tool_config_selector.current_item_changed.connect(
            self._on_tool_config_selected_callback
        )
        self._tool_config_selector.refresh_clicked.connect(
            self._on_refresh_tool_configs_callback
        )
        # Connect run_tool_config button
        self._run_button.clicked.connect(self._on_run_button_clicked)

    def add_tool_config_items(self, tool_config_names):
        '''Add given tool_configs to tool config selector'''
        for tool_config_name in tool_config_names:
            self._tool_config_selector.add_item(tool_config_name)

    def _on_context_selected_callback(self, context_id):
        '''Emit signal with the new context_id'''
        if not context_id:
            return
        self.selected_context_changed.emit(context_id)

    def _on_tool_config_selected_callback(self, item_text):
        '''Emit signal with the new selected tool config'''
        if not item_text:
            return
        self.clear_tool_config_ui()
        self.selected_tool_config_changed.emit(self.selected_tool_config_name)

    def _on_refresh_tool_configs_callback(self):
        '''Clean up tool_configs and emit signal to refresh them'''
        self._tool_config_selector.clear_items()
        self.clear_tool_config_ui()
        self.selected_tool_config_changed.emit(None)
        self.refresh_tool_configs_clicked.emit()

    def _on_run_button_clicked(self):
        '''Emit signal of run button.'''
        self.run_button_clicked.emit()

    def clear_tool_config_ui(self):
        '''Remove all widgets from the tool_config_widget layout'''
        recursive_clear_layout(self._tool_config_widget.layout())
